@@ -3,13 +3,13 @@ from __future__ import absolute_import, print_function
 import os, sys
 from os.path import dirname
 
-__version__ = '0.7.11'
+__version__ = '0.7.13'
 
 def detect_mode():
     try: import google.appengine
     except ImportError: pass
     else:
-        if os.environ.get('SERVER_SOFTWARE', '').startswith('Development'):
+        if os.getenv('SERVER_SOFTWARE', '').startswith('Development'):
             return 'GAE-LOCAL'
         return 'GAE-SERVER'
 
@@ -22,6 +22,9 @@ def detect_mode():
     if not hasattr(main, '__file__'): # console
         return 'INTERACTIVE'
 
+    if os.getenv('IPYTHONENABLE', '') == 'True':
+        return 'INTERACTIVE'
+
     if getattr(main, 'INTERACTIVE_MODE_AVAILABLE', False): # pycharm console
         return 'INTERACTIVE'
 
